#include "AAPlatform.h"
#include "Actor.h"
#include "utility.h"
#include "MessageManager.h"
#include "ActorFactory.h"
#include <iostream>
#include <vector>

using namespace std;

//OpenThreads::Barrier bar;

AAPlatform* AAPlatform::pinstance = 0;// initialize pointer

AAPlatform* AAPlatform::Instance () 
{
cout<<pinstance;
    if (pinstance == 0)  // is it the first call?
    {  
	
      pinstance = new AAPlatform(); // create sole instance
    }
    return pinstance; // address of sole instance
}

AAPlatform::AAPlatform()
{

}

void AAPlatform::startup(string propertyFileName)
{
//	int i;
	pr.readFile(propertyFileName);

	cout << "Using platform name: " << pr.getPlatformName() << endl;

	cout << "Creating startup actors..." << endl;
	
	//create the coordinator if requested
	if(pr.getCreateCoordinator())
	{
		cout << "Creating the coordinator..." << endl;
		create("Coordinator");
	}

	//create the cr
	cout << "Creating the coordinator representative..." << endl;
	create(pr.getCoordinatorRepresentative());

	//get the list of roles
	vector<string> roleVec;
	pr.getRoles(roleVec);

	//create the roles
	for(unsigned i = 0; i < roleVec.size(); i++)
	{
		if(roleVec[i] != "")
		{
			cout << "Creating role " << roleVec[i] << "..." << endl;
			create(roleVec[i]);
		}
	}

	//get the list of rr's
	vector<string> rrVec;
	pr.getRoleRepresentatives(rrVec);

	//create the rr's
	for(unsigned i = 0; i < rrVec.size(); i++)
	{
		if(rrVec[i] != "")
		{
			cout << "Creating role representative " << rrVec[i] << "..." << endl;
			create(rrVec[i]);
		}
	}

	//get the list of actors
	vector<string> actVec;
	pr.getActors(actVec);

	//create the actors
	for(unsigned i = 0; i < actVec.size(); i++)
	{
		if(actVec[i] != "")
		{
			cout << "Creating actor " << actVec[i] << "..." << endl;
			create(actVec[i]);
		}
	}
}

AAPlatform::~AAPlatform()
{

}

ActorName AAPlatform::create (string actorName)
{
	return ActorFactory::Instance()->CreateActor(actorName);
}

void AAPlatform::send (string actorName,string operation)
{
	MessageManager::Instance()->send("AAPlatform",actorName, operation);
}

string AAPlatform::getPlatformName()
{
	return pr.getPlatformName();
}

string AAPlatform::getNamingService()
{
	return pr.getNamingService();
}

string AAPlatform::getCoordinatorRepresentative()
{
	return pr.getCoordinatorRepresentative();
}

string AAPlatform::getRR(string roleName)
{
	//stripping the "Role" suffix from the role name
	int pos = roleName.length() - 4;
	string strippedName = roleName.erase(pos);

	switch(pr.getMode())
	{
	case ARC_MODE_FDM:
		return strippedName + "RRFDM";

	case ARC_MODE_PDM:
		return strippedName + "RRPDM";

	case ARC_MODE_FCM:
		return strippedName + "RRFCM";

	default:
		return roleName;
	}
}

string AAPlatform::getCR()
{
	switch(pr.getMode())
	{
	case ARC_MODE_FDM:
		return "uan:CRFDM:" + pr.getPlatformName() + ":1";

	case ARC_MODE_PDM:
		return "uan:CRPDM:" + pr.getPlatformName() + ":1";

	case ARC_MODE_FCM:
		return "uan:CRFCM:" + pr.getPlatformName() + ":1";

	default:
		return "CR";
	}
}

int AAPlatform::getMode()
{
	return pr.getMode();
}

