
#include "ActorFactoryThread.h"

ActorFactoryThread::ActorFactoryThread() 
	{
		pthread_mutex_init(&pDoneMutex, NULL);
		setDone(false);
	}

ActorFactoryThread::~ActorFactoryThread()
{
}

void ActorFactoryThread::setDone(bool value)
{
	//doneMutex.lock();
	pthread_mutex_lock(&pDoneMutex);
	done = value;
	pthread_mutex_unlock (&pDoneMutex);			
	//doneMutex.unlock();
}


bool ActorFactoryThread::isDone()
{
	//doneMutex.lock();
	pthread_mutex_lock(&pDoneMutex);
	bool value = done;
	//doneMutex.unlock();
	pthread_mutex_unlock (&pDoneMutex);	
	return value;
}

CORBA::ORB_var ActorFactoryThread::getORB()
{
		return orb;
}

PortableServer::POA_var ActorFactoryThread::getPOA()
{
		return poa;
}

CosNaming::NamingContext_var ActorFactoryThread::getNameService()
{
		return namingRoot;
}



void ActorFactoryThread::run()
{
		int argc = 3;
		char *argv[3];
		argv[0] = "AAPlatform.exe";
		argv[1] = "-ORBInitRef";
		//argv[2] = "NameService=corbaloc:iiop:x2:2809/NameService";
		argv[2] = Utility::stringToChar(AAPlatform::Instance()->getNamingService());

		try
		{
			orb = CORBA::ORB_init(argc,argv);
			cout << "ORB initialized." << endl;

			CORBA::Object_var poaObj = orb->resolve_initial_references("RootPOA");
			poa = PortableServer::POA::_narrow(poaObj.in());
			poa->the_POAManager()->activate();
			cout << "POA activated." << endl;

			CORBA::Object_var namingObj = orb->resolve_initial_references("NameService");
			namingRoot = CosNaming::NamingContext::_narrow(namingObj.in());
			cout << "Name Service found." << endl;

			CosNaming::Name name;
			name.length(1);
			name[0].id = CORBA::string_dup("example");
			try
			{
				CORBA::Object_var test = namingRoot->resolve(name);
				cout << "Naming context detected." << endl;
			}
			catch(const CosNaming::NamingContext::NotFound &)
			{
				CosNaming::NamingContext_var test = namingRoot->bind_new_context(name);
				cout << "Naming context created." << endl;
			}

			ORBThreadPool task(orb.in());
			int nthreads = 10;
			int retval = task.activate(THR_NEW_LWP | THR_JOINABLE, nthreads);
			if (retval != 0)
			{
				cerr << "Failed to activate " << nthreads << " threads!" << endl;
			}
			
			setDone(true);
			task.wait();
			orb->run();
			orb->destroy();
		}
		catch(CORBA::Exception& ex)
		{
			cerr << "ActorFactoryThread CORBA Exception: " << ex << endl;
		}
 }   

