#include "MessageManager.h"
#include "MessengerC.h"
#include "ActorCache.h"
#include <iostream>
#include "RoleRecruiter.h"
#include "Criteria.h"
//#include <CosNamingC.h>
//#include "MessengerI.h"
#include "UANManager.h"
#include "utility.h"
#include "AAPlatform.h"

using namespace std;

MessageManager* MessageManager::pinstance = 0;

MessageManager::MessageManager()
{
	int argc = 3;
	char *argv[3];
	argv[0] = "./PlatformRun";
	argv[1] = "-ORBInitRef";
	//argv[2] = "NameService=corbaloc:iiop:x2:2809/NameService";
	argv[2] = Utility::stringToChar(AAPlatform::Instance()->getNamingService());

	try {
		//Iniitialize the ORB.
		orb = CORBA::ORB_init(argc, argv);
		
		/*CORBA::Object_var poaObj = orb->resolve_initial_references("RootPOA");
		PortableServer::POA_var poa = PortableServer::POA::_narrow(poaObj.in());
		poa->the_POAManager()->activate();*/

		//Find the Naming Service
		CORBA::Object_var naming_obj =
			orb->resolve_initial_references("NameService");
		//cout << "NameService IOR " << orb->object_to_string(naming_obj) << endl;
		
		root = CosNaming::NamingContext::_narrow(naming_obj.in());
		if(CORBA::is_nil(root.in())) {
			std::cerr << "Nil Naming Context reference" << std::endl;
		}

		//clearing the cache
		//cache.clear();
	}
	catch (CORBA::Exception& ex) {
		cerr << "MessageManager CORBA exception: " << ex << endl;
	}
	cout << "MessageManager created." << endl;
}

MessageManager* MessageManager::Instance()
{
	if(pinstance == 0)
		pinstance = new MessageManager();

	return pinstance;
}

MessageManager::~MessageManager()
{

}

void MessageManager::send (string sender,string actorName,string message)
{
	cout << "MessageManager intercepted a message." << endl;

	//figure out the sender role
	string senderRole;
	if(sender == "" || sender=="AAPlatform")
		senderRole = "";
	else
		senderRole = action_synch("",sender,"role");
	
	//if sender is constrained
	if(senderRole != "")
	{
		//figure out the receiver role
		string receiverRole = action_synch("",actorName,"role");

		//figure out the sender role rr name
		string senderRR = AAPlatform::Instance()->getRR(senderRole);

		//prepare the event for sending
		string event = "check|" + actorName + "|" + message + "|parameters|" + sender + "|" + senderRole + "|" + receiverRole;

		//cout << "senderRR: " << senderRR << " event: " << event << endl;
		
		//send the event to the role representative
		cout << "MessageManager is sending an event to the role representative..." << endl;
		string checkResult = action_synch("",senderRR,event);

		cout << "Constraint check is complete." << endl;
		cout << "MessageManager is sending the message to the receiver MessageManager..." << endl;
		//do whatever you want with the checkResult here
		//apply constraints or whatever
		//it may involve redirecting the message somewhere else
	}

	//send the actual message to the receiver
	action(sender,actorName, message);
}

void MessageManager::sendUnconstrained(string sender, string actorName, string message)
{
	action(sender,actorName, message);
}

string MessageManager::call (string sender,string actorName,string message)
{
	return action_synch(sender,actorName, message);
}

void MessageManager::action (string sender,string actorName, string message)
{
#ifndef MESSAGEMANAGER_DISABLELOCALCALLS
	//if it's a local actor to this aaplatform use a local call
	//instead of corba (should save some overhead
	Actor *localActor = UANManager::Instance()->getLocalActor(actorName);

//-----------------------------
//SENDING CODE FOR LOCAL ACTORS
//-----------------------------
	if(localActor != 0)
	{
		//because it's a local call, we can do the
		//constraint check for the receiving end right here

		//THE CODE INSIDE OF THIS IF STATEMENT IS DOING
		//THE CONSTRAINT CHECK FOR THE RECEIVING SIDE (LOCAL ACTORS ONLY)
		
		string senderRole;
		string receiverRole;

		cout << "MessageManager received the message." << endl;

		if(sender == "AAPlatform")
		{
			senderRole = "";
			receiverRole = "";
		}
		else
		{
			//figure out the sender role
			if(sender == "")
				senderRole = "";
			else
				senderRole = action_synch("",sender,"role");

			//figure out the receiver role
			receiverRole = action_synch("",actorName,"role");
		}

		//if receiver is constrained
		if(receiverRole != "")
		{
			//figure out the role representative
			string receiverRR = AAPlatform::Instance()->getRR(receiverRole);

			//prepare the event for sending
			string event = "check|" + actorName + "|" + message + "|parameters|" + sender + "|" + senderRole + "|" + receiverRole;

			//cout << "receiverRR: " << receiverRR << " event: " << event << endl;

			string checkResult = action_synch("",receiverRR,event);

			//do whatever you want with the constraint check result
			//the stuff to handle it goes here
			
			cout << "Constraint check is complete." << endl;
		}

		//call the actual actor
		cout << "Message is being sent to the actor for processing..." << endl;

		localActor->handle(sender,message);
		return;
	}	
#endif

//------------------------------
//SENDING CODE FOR REMOTE ACTORS
//------------------------------
	ActorCache *actorCache = ActorCache::Instance();
	
	string senderRole;
	if(sender == "AAPlatform")
	{
		senderRole = "";
	}
	else
	{
		//figure out the sender role
		if(sender == "")
			senderRole = "";
		else
			senderRole = action_synch("",sender,"role");
	}

	try
	{
		// Send a message the Messenger object.
		//CORBA::String_var msg = CORBA::string_dup(message);

		// Narrow the IOR to a Messenger object reference.
		Messenger_var messenger;

		//check if the actor's in the cache
#ifndef MESSAGEMANAGER_DISABLEOPTIMIZATION
		if(actorCache->get(actorName,messenger))
		{
			//cout << "Cache Hit" << endl;
		}
		else
		{
#endif
			//cout << "Cache Miss" << endl;
			//Resolve the Messenger object
			CosNaming::Name name;
			name.length(2);
			name[0].id = CORBA::string_dup("example");
			name[1].id = CORBA::string_dup(actorName.c_str());
			cout << "Trying to resolve " << actorName << endl;

			CORBA::Object_var obj = root->resolve (name);
			messenger = Messenger::_narrow(obj.in());

			if (CORBA::is_nil(messenger.in())){
				cerr << "IOR was not a Messenger object reference." << endl;
				return;
			}

			//cout << "Resolved IOR " << endl << orb->object_to_string(obj);

#ifndef MESSAGEMANAGER_DISABLEOPTIMIZATION
			//populate the cache
			actorCache->put(actorName,messenger);
		}	
#endif

		string taggedMessage = senderRole + '|' + message;

		messenger->asynchronous_message(actorName.c_str(), sender.c_str(), taggedMessage.c_str());

		//Print the Messenger's reply.
		//cout << "Reply: " << msg.in() << endl;

	}
	catch (CORBA::Exception& ex) {
		cerr << "MessageManager.action() CORBA exception: " << ex << endl;
	}
}

string MessageManager::action_synch (string sender,string actorName, string message)
{
#ifndef MESSAGEMANAGER_DISABLELOCALCALLS
	//if it's a local actor to this aaplatform use a local call
	//instead of corba (should save some overhead
	Actor *localActor = UANManager::Instance()->getLocalActor(actorName);
	if(localActor != 0)
	{
		return localActor->handle_synch(sender,message);
	}	
#endif

	ActorCache *actorCache = ActorCache::Instance();

	try
	{
		// Send a message the Messenger object.
		//CORBA::String_var msg = CORBA::string_dup(message);

		// Narrow the IOR to a Messenger object reference.
		Messenger_var messenger;

		//check if the actor's in the cache
#ifndef MESSAGEMANAGER_DISABLEOPTIMIZATION
		if(actorCache->get(actorName,messenger))
		{
			//cout << "Cache Hit" << endl;
		}
		else
		{
#endif
			//cout << "Cache Miss" << endl;
			//Resolve the Messenger object
			CosNaming::Name name;
			name.length(2);
			name[0].id = CORBA::string_dup("example");
			name[1].id = CORBA::string_dup(actorName.c_str());
			cout << "Trying to resolve " << actorName << endl;

			CORBA::Object_var obj = root->resolve (name);
			messenger = Messenger::_narrow(obj.in());

			if (CORBA::is_nil(messenger.in())){
				cerr << "IOR was not a Messenger object reference." << endl;
				return "";
			}
#ifndef MESSAGEMANAGER_DISABLEOPTIMIZATION
			//populate the cache
			actorCache->put(actorName,messenger);
		}	
#endif
		//messenger->send_message(actorName, message, msg.in());
		return messenger->synchronous_message(actorName.c_str(), sender.c_str(), message.c_str());

		//Print the Messenger's reply.
		//cout << "Reply: " << msg.in() << endl;

	}
	catch (CORBA::Exception& ex) {
		cerr << "MessageManager.action_synch() CORBA exception: " << ex << endl;
		cerr << "Sender: " << sender << " ActorName: " << actorName << " Msg: " << message << endl;
		return "";
	}
}
